--[[-----------------------------------------------------------------------------
HeadCount Boss Item Widget
-------------------------------------------------------------------------------]]
local Type, Version = "HeadCountRaidItem", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, select, pairs = math.max, select, pairs

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent


--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		-- height is set dynamically by the text and image size
		self:SetNumber()
		self:SetDate()
		self:SetZone()
		self:SetSelected(false)
	end,

	-- ["OnRelease"] = nil,

	["OnWidthSet"] = function(self, width)
	end,

	["SetNumber"] = function(self, number)
		self.rowNumber = number
		self.numLabel:SetText(number)
	end,

	["SetDate"] = function(self,date)
		self.dateLabel:SetText(date)
	end,

	["SetBoss"] = function(self,zone)
		self.nameLabel:SetText(zone)
	end,

	["SetMembers"] = function(self,zone)
		self.memberLabel:SetText(zone)
	end,

	["SetSelected"] = function(self, value)
		if value then
			self.selectedRow:Show()
		else
			self.selectedRow:Hide()
		end
	end,

	["SetRaidID"] = function(self, value)
		self.raidID = value
	end,

}

local function HitArea_OnEnter(frame)
	frame.obj.highlight:Show()
end

local function HitArea_OnClick(frame)
	frame.obj:Fire("BossSelected",frame.obj.raidID, frame.obj.rowNumber)
end

local function HitArea_OnLeave(frame)
	GameTooltip:Hide()
	frame.obj.highlight:Hide()
end

local function Delete_OnClick(frame)
	frame.obj:Fire("BossDeleted",frame.obj.raidID)
end
local function Button_OnLeave(frame)
	GameTooltip:Hide()
end

local function Button_OnEnter(frame)
	if frame.tt then
		GameTooltip:SetOwner(frame)
		GameTooltip:AddLine(frame.tt)
		GameTooltip:Show()
	end
end

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:Hide()
	frame:SetWidth(596)
	frame:SetHeight(18)

	local numLabel = frame:CreateFontString(nil,"ARTWORK", "GameFontNormalSmall")
	numLabel:SetWidth(28)
	numLabel:SetHeight(16)
	numLabel:SetJustifyH("LEFT")
	numLabel:SetTextColor(1,1,1,1)
	numLabel:SetPoint("TOPLEFT",8,0)

	local dateLabel = frame:CreateFontString(nil,"ARTWORK","GameFontNormalSmall")
	dateLabel:SetWidth(124)
	dateLabel:SetHeight(16)
	dateLabel:SetJustifyH("LEFT")
	dateLabel:SetTextColor(1,1,1,1)
	dateLabel:SetPoint("TOPLEFT",38,0)

	local memberLabel = frame:CreateFontString(nil,"ARTWORK","GameFontNormalSmall")
	memberLabel:SetWidth(28)
	memberLabel:SetHeight(16)
	memberLabel:SetJustifyH("LEFT")
	memberLabel:SetTextColor(1,1,1,1)
	memberLabel:SetPoint("TOPLEFT",162,0)

	local nameLabel = frame:CreateFontString(nil,"ARTWORK","GameFontNormalSmall")
	nameLabel:SetWidth(160)
	nameLabel:SetHeight(16)
	nameLabel:SetJustifyH("LEFT")
	nameLabel:SetTextColor(1,1,1,1)
	nameLabel:SetPoint("TOPLEFT",162,0)

	local mouseOverTexture = frame:CreateTexture(nil,"BACKGROUND")
	mouseOverTexture:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
	mouseOverTexture:SetWidth(583)
	mouseOverTexture:SetHeight(15)
	mouseOverTexture:SetPoint("TOPLEFT",6,-1)
	mouseOverTexture:SetVertexColor(0.63,0.63,1,1)
	mouseOverTexture:Hide()

	local selectedRow = frame:CreateTexture(nil,"BACKGROUND")
	selectedRow:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
	selectedRow:SetWidth(583)
	selectedRow:SetHeight(15)
	selectedRow:SetPoint("TOPLEFT",6,-1)
	selectedRow:SetVertexColor(0.63,0.63,1,1)
	selectedRow:Hide()

	local hitArea = CreateFrame("Button",nil,frame)
	hitArea:SetWidth(506)
	hitArea:SetHeight(18)
	hitArea:SetPoint("TOPLEFT")
	hitArea:SetScript("OnLeave",HitArea_OnLeave)
	hitArea:SetScript("OnEnter",HitArea_OnEnter)
	hitArea:SetScript("OnClick",HitArea_OnClick)
	hitArea:EnableMouse(true)

	local dButton = CreateFrame("Button",nil,frame)
	dButton:SetWidth(24)
	dButton:SetHeight(24)
	dButton:SetPoint("TOPLEFT",553,2)
	dButton:SetNormalTexture("Interface\\Buttons\\CancelButton-Up")
	dButton:SetPushedTexture("Interface\\Buttons\\CancelButton-Down")
	dButton:SetHighlightTexture("Interface\\Buttons\\CancelButton-Highlight","ADD")
	dButton:EnableMouse(true)
	dButton:SetScript("OnClick",Delete_OnClick)
	dButton:SetScript("OnLeave",Button_OnLeave)
	dButton:SetScript("OnEnter",Button_OnEnter)

	-- create widget
	local widget = {
		numLabel = numLabel,
		dateLabel = dateLabel,
		memberLabel = memberLabel,
		raidID = 0,
		rowNumber = 0,
		selectedRow = selectedRow,
		highlight = mouseOverTexture,
		export = eButton,
		delete = dButton,
		frame = frame,
		export_TT = "",
		delete_TT = "",
		type  = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end
	hitArea.obj = widget
	eButton.obj, dButton.obj = widget, widget
	eButton.tt = widget.export_TT
	dButton.tt = widget.delete_TT
	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
